/************************************************************************
*************************************************************************
****                                                                 ****
****                       Projet : LANCER DE RAYON                  ****
****                      fichier : fenetre.h                        ****
****                                                                 ****
*************************************************************************
*************************************************************************
* Auteurs :                               date : 12/05/05               *
*          GUITTENY Fabrice                                             *
*          LE GOFF Erwan                                                *
*          LAHUEC Morgan                                                *
*************************************************************************
* Class: Fenetre                                                        *
*                                                                       *
*************************************************************************
* Description :                                                         *
* La fenetre defini la zone que les rayons vont parcourir.              *   
* Elle correspond  ce qui sera affiche  l'cran.                      *
************************************************************************/
#ifndef _FENETRE_H
#define _FENETRE_H

#include<iostream>
#include"point.h"

using namespace std;

class Fenetre {

/* une fentre est dfinie par un point de "dpart" (correspond au coin
   haut gauche), par une largeur (taille sur (Ox)) et une hauteur (taille sur (Oy))
   --> une fentre est donc forcment parallle au plan (Oxy) ce qui simplifie le problme
   On aurait pu dfinir une fentre par 2 points correspondants respectivement aux coin 
   haut gauche et au coin bas droit de la fentre. Dans ce cas, la fentre aurait pu
   tre dans n'importe quel plan de l'espace                                               */

private :

	Point3D coinHautGauche;			// coin haut gauche de la fentre
	int largeur;				// largeur de la fentre
	int hauteur;				// hauteur de la fentre

public :

	// constructeurs
	Fenetre() {};
	Fenetre(const Fenetre &f) {coinHautGauche=f.coinHautGauche;largeur=f.largeur;hauteur=f.hauteur;};
	Fenetre(const Point3D &pCHG, const int &larg, const int &haut) {coinHautGauche=pCHG;largeur=abs(larg);hauteur=abs(haut);};

	// destructeur
	~Fenetre() {};

	// accesseurs
	Point3D getCHG() {return coinHautGauche;};
	int getLargeur() {return largeur;};
	int getHauteur() {return hauteur;};

	// modificateurs
	void setCHG(const Point3D &pCHG) {coinHautGauche=pCHG;};
	void setLargeur(const int &larg) {largeur=abs(larg);};
	void setHauteur(const int &haut) {hauteur=abs(haut);};

	// obtention du coin bas droit (fonction non utilise)
	Point3D Fenetre::coinBasDroit() {
	   float xcbd, ycbd, zcbd;
	   xcbd=coinHautGauche.getX()+(float)largeur;
	   ycbd=coinHautGauche.getY()-(float)hauteur;
	   zcbd=coinHautGauche.getZ();	  
	   Point3D cbd=Point3D(xcbd,ycbd,zcbd);
	   return cbd;
	};
	
	// affichage des infos de la fentre
	void afficherInfos() {
		cout<<"- Coin haut gauche : ";
		coinHautGauche.afficherInfos();
		cout<<"- Largeur : "<<largeur<<endl;
		cout<<"- Hauteur : "<<hauteur<<endl;
	};
};

#endif
