/************************************************************************
*************************************************************************
****                                                                 ****
****                       Projet : LANCER DE RAYON                  ****
****                      fichier : lrayons.h                        ****
****                                                                 ****
*************************************************************************
*************************************************************************
* Auteurs :                               date : 12/05/05               *
*          GUITTENY Fabrice                                             *
*          LE GOFF Erwan                                                *
*          LAHUEC Morgan                                                *
*************************************************************************
* Class:                                                                *
*                                                                       *
*************************************************************************
* Description :                                                         *
* Un rayon est un vecteur associe a un point d'origine                  *   
*                                                                       *
************************************************************************/

#ifndef _LRAYONS_H
#define _LRAYONS_H
    
#include"rayon.h"
#include<math.h>
#include<fstream>
#include"couleur.h"
#include"vecteur.h"
#include<iostream>
using namespace std;
    
Couleur lancerRayon(Scene * s,Rayon rayon,int niveau, int indice_sphere){
    
  Couleur c_pixel;
    
  //Couleur c_fond = Couleur(204,204,255);
  Couleur c_fond = Couleur(0,0,0);
  //Couleur c_fond = Couleur(255,255,255);
  
  // couleur de la sphere traitee
  Couleur couleur_sphere = Couleur();
    
  // intersection la plus proche entre un rayon et une sphre
  Point3D * intersection = NULL;
    
  // intersection intermdiaire la plus proche
  // (utile si le rayon intersecte d'autres spheres, on garde alors la plus proche)
  Point3D * intersection_temp = NULL;
    
  // sphere intersecte par le rayon
  Sphere * sph = NULL;
  int indice_sph=0;
    
  // on cherche l'intersection la plus proche avec toutes les sphres de la scne
  for ( int i = 0; i < s->getListeSpheres().size(); i++ ) {
    
    sph = s->getListeSpheres()[i];                             // sphre courante 
    intersection_temp = rayon.hitSphere(sph);                    // calcul de l'intersection :
    if((indice_sphere==-1)||(indice_sphere!=i)){                      
      if (intersection_temp!=NULL) {                             // si l'intersection n'est pas nulle,
	if (intersection==NULL) {                                // alors elle devient l'intersection la plus proche
	  intersection=intersection_temp;
	  couleur_sphere = sph->getCouleur();                 // si il n'y avait pas d'autre intersection 	    
	  indice_sph=i;
	}                                                        // ou si le z de cette intersection est suprieur
	else {                                                   // au z de l'ancienne intersection
	  if (intersection_temp->getZ()>=intersection->getZ()) {  // (le point est plus proche de l'observateur)          
	    couleur_sphere = sph->getCouleur();
	    indice_sph=i;
	    intersection=intersection_temp;
	  }
	}
      }
    }
  }
          	
  if (intersection!=NULL) {                        // on a trouv un point d'intersection
    sph = s->getListeSpheres()[indice_sph];        // sphre courante
    Vecteur3D vect_normal = Vecteur3D();
   
 // on cherche la vritable intersection si la sphre est triangule
    bool estIntersectee = true;
    if (sph->estTriangulee()) {
      Rayon ray_tri=rayon.hitSphereTriangulee(sph);
      if((ray_tri.getOrigine()==Point3D(0.f,0.f,0.f))&&(ray_tri.getVectDir()==Vecteur3D(0.f,0.f,0.f)))
	intersection = NULL;
      else {
	intersection = new Point3D(ray_tri.getOrigine());
	vect_normal=ray_tri.getVectDir();
      }

      estIntersectee = (intersection!=NULL);
    }
    
    if (estIntersectee) {

      // creation des rayons reflechis
      Rayon r_reflechi = Rayon();
      // origine du rayon = point d'intersection
      r_reflechi.setOrigine(*intersection);
      // lumiere ambiante
      float indAmbiant = 1-sph->getIndMiroir()-sph->getIndDiffusion()-sph->getIndSpeculaire();
      // intensit ambiante (fixe dans le programme)
      float intensiteAmbiante = 1.f;
      // lumiere diffuse
      float lum_diffuse_rouge = 0.f;
      float lum_diffuse_verte = 0.f;
      float lum_diffuse_bleue = 0.f;
    
      float somme_contributions = 0.0f;
      float somme_spec = 0.0f;
    	
      float Cs_r = 0.0f;
      float Cs_v = 0.0f;
      float Cs_b = 0.0f;
    
      if (!(sph->estTriangulee())) {
	vect_normal = Vecteur3D(sph->getCentre(),*intersection);
      }
      // parcours de toutes les sources lumineuses
      for ( int j = 0; j < s->getListeSrcLumiere().size(); j++) {
	Vecteur3D vect_reflechi = Vecteur3D(*intersection,s->getListeSrcLumiere()[j].getPSource());
	// cration du rayon rflchi
	r_reflechi.setVectDir(vect_reflechi);
	// on cherche si le point est cach par une autre sphre
	Point3D * inter = NULL; 
	int k=0;
	Sphere * sphere = NULL;
	bool cache = false;
	while ((!cache) && (k < s->getListeSpheres().size())) {
	  sphere = s->getListeSpheres()[k]; 
	  if (k!=indice_sph) {  
	    if (sphere->estTriangulee()) {
	      Rayon ray_tria = r_reflechi.hitSphereTriangulee(sphere);
	      if((ray_tria.getVectDir() == Vecteur3D(0.f,0.f,0.f)) && (ray_tria.getOrigine()==Point3D(0.f,0.f,0.f)))
		inter = NULL;
	      else inter = new Point3D(ray_tria.getOrigine());
	    }

	    else {
	      inter = r_reflechi.hitSphere(sphere);
	    }
	    if (inter!=NULL) 
	      cache = true;
	  }
	  k++;
	}
	if (!cache) {
	  // creation du vecteur normal
	  float produit_scalaire = 0;
	  // produit scalaire entre la normale et le rayon rflchi
	  produit_scalaire = vect_normal.Normalise()*vect_reflechi.Normalise();
	  // si le produit scalaire est positif, la source contribue  l'clairement du point
	  if (produit_scalaire>=0) {
	    somme_contributions += produit_scalaire * s->getListeSrcLumiere()[j].getIntensite();
    
	    Vecteur3D vect_spec = (vect_normal.Normalise() * (2 * produit_scalaire)) - vect_reflechi.Normalise();
	    Vecteur3D vect_s = Vecteur3D(*intersection,rayon.getOrigine());
	    Couleur coul_src = s->getListeSrcLumiere()[j].getCouleur();
	    int rugosite = sph->getIndRugosite();
	    somme_spec = pow(s->getListeSrcLumiere()[j].getIntensite() * (vect_spec.Normalise() * vect_s.Normalise()),rugosite);
	    Cs_r += coul_src.getRouge() * somme_spec;
	    Cs_v += coul_src.getVert() * somme_spec;
	    Cs_b += coul_src.getBleu() * somme_spec;
	  }
	}
      }	
      float Ca_r = (float)couleur_sphere.getRouge() * intensiteAmbiante;
      float Ca_v = (float)couleur_sphere.getVert() * intensiteAmbiante;
      float Ca_b = (float)couleur_sphere.getBleu() * intensiteAmbiante;
    
      float Cd_r = (float)couleur_sphere.getRouge() * somme_contributions;
      float Cd_v = (float)couleur_sphere.getVert() * somme_contributions;
      float Cd_b = (float)couleur_sphere.getBleu() * somme_contributions;
    
      lum_diffuse_rouge = indAmbiant * Ca_r + sph->getIndDiffusion() * Cd_r + sph->getIndSpeculaire() * Cs_r;
      lum_diffuse_verte = indAmbiant * Ca_v + sph->getIndDiffusion() * Cd_v + sph->getIndSpeculaire() * Cs_v;
      lum_diffuse_bleue = indAmbiant * Ca_b + sph->getIndDiffusion() * Cd_b + sph->getIndSpeculaire() * Cs_b;
    		
      c_pixel =  Couleur((int)lum_diffuse_rouge,(int)lum_diffuse_verte,(int)lum_diffuse_bleue);

      if((sph->getIndMiroir()>0)&&(niveau<5)) {

	niveau++;
          
	float i_scalaire_n = rayon.getVectDir().Normalise() * vect_normal.Normalise();
	Vecteur3D rayM = (vect_normal.Normalise()*(-2*i_scalaire_n)) + rayon.getVectDir().Normalise();
	Rayon reflet = Rayon(*intersection,rayM.Normalise());

	Couleur c_pixel_int = lancerRayon(s,reflet,niveau,indice_sph);

	if (!(c_pixel_int == c_fond)) {
	  int r_pix = (int)(c_pixel_int.getRouge() * sph->getIndMiroir()+(c_pixel.getRouge()*(1-sph->getIndMiroir())));
	  int v_pix = (int)(c_pixel_int.getVert() * sph->getIndMiroir()+(c_pixel.getVert()*(1-sph->getIndMiroir())));
	  int b_pix = (int)(c_pixel_int.getBleu() * sph->getIndMiroir()+(c_pixel.getBleu()*(1-sph->getIndMiroir())));

	  c_pixel = Couleur(r_pix,v_pix,b_pix);
	}
      }
    }
              
    else {                     // pas d'intersection --> couleur du fond
      c_pixel = Couleur(c_fond);
    }
  }

  else {
    c_pixel = Couleur(c_fond);
  }
      
  if (c_pixel.getRouge() >255) c_pixel.setRouge(255);
  if (c_pixel.getVert() >255) c_pixel.setVert(255);
  if (c_pixel.getBleu() >255) c_pixel.setBleu(255);
  return c_pixel;
}
    
    
    
    
void affiche(Scene * s, char * xmlfic) {
        
  // pas de balayage
  float pas = 0.2f;
  float inv_pas = 1/pas;
  // on rcupre la taille de la fentre
  int largeur = s->getFenetre().getLargeur();
  int hauteur = s->getFenetre().getHauteur();
    
  float cent_percent = largeur*hauteur*inv_pas;
  // cration du fichier PPM
  ofstream fic_PPM;
  int size = strlen(xmlfic);
  xmlfic[size-3] = 'p';
  xmlfic[size-2] = 'p';
  xmlfic[size-1] = 'm';
  fic_PPM.open(xmlfic,ios::out);             // nom du fichier PPM de sortie
  fic_PPM << "P3" << endl;                        // format du PPM
  fic_PPM << "#" << xmlfic << endl;            
  fic_PPM << (int)inv_pas * largeur+1 << " " << (int)inv_pas * hauteur+1 << endl;   // on crit la taille du PPM
  fic_PPM << 255 << endl;                         // le nombre de bits pour les couleurs
      
  // rcupration du coin haut gauche de la fentre
  Point3D origine_fenetre = s->getFenetre().getCHG();
    
  // les coordonnes de ce point nous donnent les coordonnes de dpart du balayage
  float depart_X = origine_fenetre.getX();
  float depart_Y = origine_fenetre.getY();
    
  // les coordonnes d'arrive sont :
  float arrivee_X = depart_X + (float)largeur;
  float arrivee_Y = depart_Y + (float)hauteur;
      
  // cration d'un rayon
  Rayon ray = Rayon();
  float z_observateur = s->getPosObservateur();
  Point3D obs = Point3D(0,0,z_observateur);          // l'origine du rayon est le point (0,0,z_observateur)
  ray.setOrigine(obs);
    
  // z des vecteurs = z_fenetre - z_observateur
  float z_fenetre = s->getFenetre().getCHG().getZ();
  float z_vect = z_fenetre - z_observateur;
      
  Vecteur3D vect;
      
  cout<<"Ray Tracing in progress..."<<endl;
  // balayage de la fentre
  float y,x;
  Couleur coul_pixel;
  for ( y = depart_Y; y < arrivee_Y; y+=pas) {
    for ( x = depart_X; x < arrivee_X; x+=pas ) { 
      vect = Vecteur3D(x,y,z_vect);
      ray.setVectDir(vect.Normalise());
      coul_pixel = lancerRayon(s,ray,0,-1);
      fic_PPM<<coul_pixel.getRouge()<<" "<<coul_pixel.getVert()<<" "<<coul_pixel.getBleu()<<endl;
    }
  }
  fic_PPM.close();
  cout<<"Ray Tracing finished"<<endl;
}
#endif
