/************************************************************************
*************************************************************************
****                                                                 ****
****                       Projet : LANCER DE RAYON                  ****
****                      fichier : rayon.cpp                        ****
****                                                                 ****
*************************************************************************
*************************************************************************
* Auteurs :                               date : 12/05/05               *
*          GUITTENY Fabrice                                             *
*          LE GOFF Erwan                                                *
*          LAHUEC Morgan                                                *
*************************************************************************
* Class: Rayon                                                          *
*                                                                       *
*************************************************************************
* Description :                                                         *
* Un rayon est un vecteur associe a un point d'origine                  *   
*                                                                       *
************************************************************************/

#include"rayon.h"

// intersection entre un rayon et une sphre parfaite
// -> on renvoie le point d'intersection

Point3D* Rayon::hitSphere (Sphere * s) {

  // centre de la sphere
  Point3D centreS=s->getCentre();

  // rayon de la sphere
  float R=s->getRayon();
  
  // coordonnes du centre de la sphre
  float x0=centreS.getX();
  float y0=centreS.getY();
  float z0=centreS.getZ();
  
  // coordonnes du vecteur du rayon
  float Vx=vectDir.getX();
  float Vy=vectDir.getY();
  float Vz=vectDir.getZ();
  
  // paramtre
  float t=-1.f;

  // coordonnes de l'origine du rayon
  float Fx=origine.getX();
  float Fy=origine.getY();                  
  float Fz=origine.getZ();

  // quation :   t*(Vx+Vy+Vz)
  //            + t*2*(VxFx+VyFy+VzFz-Vx.x0-Vy.y0-Vz.z0)
  //            + Fx+Fy+Fz+x0+y0+z0-2Fx.x0-2Fy.y0-2Fz.z0-R
  //            = 0

  // observateur fix sur l'axe des z --> Fx et Fy =0
  // float a=Vx*Vx+Vy*Vy+Vz*Vz;	
  // float b=2*(Vz*Fz-Vx*x0-Vy*y0-Vz*z0);
  // float c=Fz*Fz+x0*x0+y0*y0+z0*z0-2*Fz*z0-R*R;
  
  // coefficents de l'quation  
  float a=Vx*Vx+Vy*Vy+Vz*Vz;
  float b=2*(Vx*Fx+Vy*Fy+Vz*Fz-Vx*x0-Vy*y0-Vz*z0);
  float c=Fx*Fx+Fy*Fy+Fz*Fz+x0*x0+y0*y0+z0*z0-2*Fx*x0-2*Fy*y0-2*Fz*z0-R*R;
  
  // discriminant de l'quation
  float delta=b*b-4*a*c;

  // coordonnes du point d'intersection
  float xint;
  float yint;
  float zint;
  
  // point rsultat
  Point3D * res = NULL;
  
  // dtermination du nombre de points d'intersection
  
  if (delta==0.f) {                 		// un seul point d'intersection avec la sphere
    t=(-b)/(2*a);
    xint=Fx+t*Vx;
    yint=Fy+t*Vy;
    zint=Fz+t*Vz;
    res=new Point3D(xint,yint,zint);   
  }
  else {
    if (delta>0.f) {                  	 	// 2 points d'intersection --> on cherche le t minimal
      float t1 = (-b+sqrt(delta))/(2*a);	
      float t2 = (-b-sqrt(delta))/(2*a);
      if ((t1<=0.f) || (t2<=0.f))
	t=max(t1,t2);
      else
	t=min(t1,t2);
      xint=Fx+t*Vx;
      yint=Fy+t*Vy;
      zint=Fz+t*Vz;
      res=new Point3D(xint,yint,zint);
    }
  }
 
  // on retourne le point trouv si seulement t est >=0
  if (t>=0.f)
    return res;
  else 
    return NULL;
};



// intersection entre un rayon et une sphre triangule
// -> on renvoie un rayon dont l'origine est le point d'intersection
//    et le vecteur directeur est le vecteur normal  la facette triangulaire intersecte
  
Rayon Rayon::hitSphereTriangulee (Sphere * s) {

  Vecteur3D res = Vecteur3D(0.f,0.f,0.f);			// vecteur normal  la facette
  Point3D intersection = Point3D(0.f,0.f,0.f) ;			// point d'intersection
  vector<Triangle*> ensTriangles = s->getListeTriangles();	// ensemble des triangles de la sphre
  int nbTriangles = ensTriangles.size();			// nombre de triangles de la sphre 
  int i=0;
  bool pt_valide = false;					// boolen permettant de savoir si une intersection trouve est valide ou non
  Triangle * tri = NULL;
  Point3D A, B, C;
  Vecteur3D V1, V2;
  float x1, y1, z1, x2, y2, z2;
  float xA, yA, zA;
  float x3 = vectDir.getX(), y3 = vectDir.getY(), z3 = vectDir.getZ();	// coordonnes du vecteur directeur du rayon
  float xP = origine.getX(), yP = origine.getY(), zP = origine.getZ();	// coordonnes de l'origine du rayon
  float x, y, z;							// coordonnes de l'intersection
  float D=0.f, u=0.f, v=0.f, t=100000000.f, tint=0.f;			// coefficients du systme  rsoudre
  for (i=0; i<nbTriangles; i++) {					// parcours de tous les triangles
    // rcupration d'un triangle et de ses 3 points
    tri = ensTriangles[i];
    A = tri->getPointA(); B = tri->getPointB(); C = tri->getPointC();
    xA = A.getX(); yA = A.getY(); zA = A.getZ();
    // vecteurs AB et AC
    V1 = Vecteur3D(A,B); V2 = Vecteur3D(A,C);
    // coordonnes de ces vecteurs
    x1 = V1.getX(); y1 = V1.getY(); z1 = V1.getZ();
    x2 = V2.getX(); y2 = V2.getY(); z2 = V2.getZ();
    // rsolution du systme
    // (quations trouves sur Internet...)
    D = -z1*y3*x2 + y1*z3*x2 + x3*z1*y2 + y3*x1*z2 - z3*x1*y2 - x3*y1*z2;
    u = -(-y3*x2*zA - y3*xP*z2 + y3*x2*zP + y3*xA*z2 + x3*y2*zA - x3*y2*zP + x2*z3*yA - x2*z3*yP
	  + xP*z3*y2 + x3*yP*z2 - x3*yA*z2 - xA*z3*y2)/D;

    v = (-x3*z1*yA - x3*y1*zP + x3*z1*yP + x3*y1*zA + z3*x1*yA + y1*z3*xP + z1*y3*xA - z3*x1*yP
	 - y3*x1*zA - y1*z3*xA + y3*x1*zP - z1*y3*xP)/D;

    tint = (-xP*z1*y2 + xP*y1*z2 + xA*z1*y2 - xA*y1*z2 - x1*y2*zA + x1*y2*zP - x1*yP*z2 + x1*yA*z2
	    - x2*z1*yA - x2*y1*zP + x2*z1*yP + x2*y1*zA)/D;
    
    // conitions pour que l'intersection soit valide
    pt_valide = ((D!=0) && (u>0) && (v>0) && (u+v<=1));
    
    // si l'intersection trouve est valide et qu'elle est plus proche que l'intersection prcdente,
    // on garde le paramtre t correspondant et le vecteur normal  la facette du point
    if (pt_valide && (tint<t) && (tint>=0)) {
      t = tint;
      res = (V1^V2);
    }
  }

  // aprs avoir parcouru tous les triangles, si on a trouv une intersection alors on renvoie le rayon correspondant (intersection + vecteur normal)
  // sinon on renvoie le rayon : origine = (0,0,0) et vecteur directeur = (0,0,0)
  
  if (t!=100000000.f) {
    x = x3 * t + xP;
    y = y3 * t + yP;
    z = z3 * t + zP;
    intersection = Point3D(x,y,z);
  }
  else
    intersection = Point3D(0.f,0.f,0.f);
  
  if (res==Vecteur3D(0.f,0.f,0.f))
    return Rayon(intersection,res);
  else
    return Rayon(intersection,res.Normalise());
};
