/************************************************************************
*************************************************************************
****                                                                 ****
****                       Projet : LANCER DE RAYON                  ****
****                      fichier : sphere.cpp                       ****
****                                                                 ****
*************************************************************************
*************************************************************************
* Auteurs :                               date : 12/05/05               *
*          GUITTENY Fabrice                                             *
*          LE GOFF Erwan                                                *
*          LAHUEC Morgan                                                *
*************************************************************************
* Class: Sphere                                                         *
*                                                                       *
*************************************************************************
* Description :                                                         *
* Une sphere est representee par les 3 coordonnees de son centre, les 3 *   
* composantes de sa couleur, son rayon, son indice miroir, son indice   *
* diffusion, son indice speculaire, son indice de rugosite, son type de *
* triangulation, son indice de triangulation.                           *
************************************************************************/

#include"sphere.h"
#include"rayon.h"
#include"vecteur.h"
#include"triangle.h"

// position d'un point par rapport  une sphere
// 0 :  la surface, 1 :  l'intrieur, 2 :  l'extrieur

int Sphere::positionParRapportSphere(Point3D p) {  

  // rcupration de l'quation de la sphre
  float x=(p.getX()-centre.getX()) * (p.getX()-centre.getX());
  float y=(p.getY()-centre.getY()) * (p.getY()-centre.getY());
  float z=(p.getZ()-centre.getZ()) * (p.getZ()-centre.getZ());
  float r=rayon * rayon;
	
  if (x+y+z==r)
    return 0;		// point sur la sphre
  else
    if (x+y+z<r)
      return 1;	// point  l'intrieur de la sphre
    else
      return 2;	// point  l'extrieur de la sphre
};


// affichage des infos de la sphere

void Sphere::afficherInfos() {
  cout<<"- Centre : ";
  centre.afficherInfos();
  cout<<"- Rayon : "<<rayon<<endl;
  cout<<"- Indice miroir : "<<indMiroir<<endl;
  cout<<"- Indice de diffusion : "<<indDiffusion<<endl;
  cout<<"- Indice spculaire : "<<indSpeculaire<<endl;
  cout<<"- Indice de rugosit : "<<indRugosite<<endl;
  cout<<"- Couleur : ";
  couleur.afficherInfos();
  switch(triangulee){
  case 1 : cout<<"triangulation type 1 "<<"indice de triangulation : "<<indTriangulee<<endl;break;
  case 2 : cout<<"triangulation type 2 "<<"indice de triangulation : "<<indTriangulee<<endl;break;
  default : cout<<"sphere parfaite"<<endl; break;
  }
};


// triangulation d'une sphre parfaite

vector<Triangle*> Sphere::Triangule() {
  vector<Triangle*> res;
  float pi = 3.1415926535;
  float ecart_OE = indTriangulee;				// cart entre 2 lignes Ouest - Est
  float ecart_NS = ecart_OE/2;				// cart entre 2 lignes Nord - Sud
  float delta_teta = (float) (2 * pi / ecart_OE);		// diffrence entre 2 angles Ouest - Est
  float delta_phi = (float) (pi / (2 * ecart_NS));	// diffrence entre 2 angles Nord - Sud
  float teta, phi;
  float phi_pole = (float) ((ecart_NS-1) * pi / (2 * ecart_NS));
  float x,y,z;
  Point3D p1,p2,p3;
  int i,j;
  for (i=0;i<ecart_OE;i++) {
    teta = (float) (i * delta_teta);
    for (j=0;j<ecart_NS-1;j++) {
      phi = (float) (j * delta_phi);
      // cration du 1er triangle  (hmisphre nord)
      p1 = p1.engendrerPoint(teta,phi,rayon,centre);
      p2 = p2.engendrerPoint(teta,phi+delta_phi,rayon,centre);
      p3 = p3.engendrerPoint(teta+delta_teta,phi+delta_phi,rayon,centre);
      res.push_back(new Triangle(p1,p2,p3,couleur));
      // cration du 2me triangle (hmisphre nord)
      p2 = p2.engendrerPoint(teta+delta_teta,phi+delta_phi,rayon,centre);
      p3 = p3.engendrerPoint(teta+delta_teta,phi,rayon,centre);
      res.push_back(new Triangle(p1,p2,p3,couleur));
      // cration du 3me triangle (hmisphre sud)
      p1 = p1.engendrerPoint(teta,-phi,rayon,centre);
      p2 = p2.engendrerPoint(teta,-phi-delta_phi,rayon,centre);
      p3 = p3.engendrerPoint(teta+delta_teta,-phi-delta_phi,rayon,centre);
      res.push_back(new Triangle(p1,p3,p2,couleur));
      // cration du 4me triangle (hmisphre sud)
      p2 = p2.engendrerPoint(teta+delta_teta,-phi-delta_phi,rayon,centre);
      p3 = p3.engendrerPoint(teta+delta_teta,-phi,rayon,centre);
      res.push_back(new Triangle(p1,p3,p2,couleur));
    }
    // cration du triangle du ple nord
    p1 = p1.engendrerPoint(teta,phi_pole,rayon,centre);
    p2 = p2.engendrerPoint(teta+delta_teta,phi_pole,rayon,centre);
    p3 = Point3D(centre.getX(),rayon+centre.getY(),centre.getZ());
    res.push_back(new Triangle(p1,p2,p3,couleur));
    // cration du triangle du ple sud
    p1 = p1.engendrerPoint(teta,-phi_pole,rayon,centre);
    p2 = p2.engendrerPoint(teta+delta_teta,-phi_pole,rayon,centre);
    p3 = Point3D(centre.getX(),-rayon+centre.getY(),centre.getZ());
    res.push_back(new Triangle(p1,p2,p3,couleur));
  }
  return res;
};


// 2me mthode de triangulation :
// --> triangulation de l'quateur
// puis projection des triangles sur la sphre

vector<Triangle*> Sphere::TrianguleEquateur() {

  vector<Triangle*> res;                                  // ensemble des triangles de la sphre
  vector<Triangle*> res_int;                              // ensemble des triangles de l'quateur
  float pi = 3.1415926535;                                // pi
  float y_equateur = centre.getY();                       // le plan de l'quateur est donn par le y du centre de la sphre
  float nb_cercles = indTriangulee;                       // indTriangulee correspond au nombre de cercles internes 
  
  int nb_triangles = 7;                                   // 7 triangles sur le 1er cercle interne, 14 sur les autres
  double angle = 0;                                       // angle en cours pour le calcul des cos et sin
  double angle_depart = 0;                                // angle de dpart (ds que l'on passe sur un nouveau cercle)
  double ecart_angles = (double) (2*pi / 7);              // cart entre 2 angles (fixe sur tous les cercles)
  double rayon_cercle = 0;                                // rayon du cercle interne en cours
  double rayon_cercle_prec = 0;                           // rayon du cercle interne prcdent
  Point3D p1,p2,p3;                                       // points du triangle
  Triangle *tri = NULL;                                   // triangle gnr
  
  double  alpha = (double) (pi/2);			  // angle permettant que les triangles soient relativement de la bonne taille
  							  // (ie que les triangles prs de l'quateur ne soient pas plus grands que les autres)
  double ecart_alpha = (double) ((pi/2)/nb_cercles);	  // cart pour l'angle alpha
  
  float x_centre = centre.getX();
  float z_centre = centre.getZ();
  
  // on engendre des triangles pour chaque cercle interne
  for(int i=0;i<nb_cercles;i++) {
  
    alpha -= ecart_alpha;				  // l'angle alpha est dcrment de ecart_alpha
      
    if (i>1)
      angle_depart += ecart_angles / 2;                   // l'angle de dpart est incrment de cart_angles/2
    // ds que l'on est sur le 2me cercle interne
    angle = angle_depart;                                 // l'angle en cours est initialis  l'angle de dpart
    rayon_cercle_prec = rayon_cercle;                     // la valeur du rayon du cercle prcdent est rayon_cercle
    rayon_cercle = (double) (cos(alpha)*rayon);           // rayon_cercle est ensuite modifi
       
    if (i!=0)                           // si on n'est pas sur le 1er cercle interne, le nombre de triangles est de 14
      nb_triangles = 14;

    for(int j=0;j<nb_triangles;) {      // on cre nb_triangles 
          
	if (i==0) {			// 1er cercle interne --> correspond aux ples donc gr diffremment
	// on gnre un triangle par un triangle (7 en tout)
	p1 = centre;
	p2 = Point3D(x_centre+rayon_cercle*cos(angle),y_equateur,z_centre+rayon_cercle*sin(angle));
	p3 = Point3D(x_centre+rayon_cercle*cos(angle+ecart_angles),y_equateur,z_centre+rayon_cercle*sin(angle+ecart_angles));
	tri = new Triangle(p1,p2,p3,couleur);
	res_int.push_back(tri);          // ajout du triangle dans la liste des triangles de l'quateur
	angle += ecart_angles;           // l'angle est incrment de ecart_angles
	j++;                             // j est incrment de 1 (nombre de triangles crs)
      }

      else {
	// les autres cercles internes --> on gnre 2 triangles par 2 triangles
	p1 = Point3D(x_centre+rayon_cercle_prec*cos(angle),y_equateur,z_centre+rayon_cercle_prec*sin(angle));
	p2 = Point3D(x_centre+rayon_cercle*cos(angle-ecart_angles/2),y_equateur,z_centre+rayon_cercle*sin(angle-ecart_angles/2));
	p3 = Point3D(x_centre+rayon_cercle*cos(angle+ecart_angles/2),y_equateur,z_centre+rayon_cercle*sin(angle+ecart_angles/2));
	tri = new Triangle(p1,p2,p3,couleur);
	res_int.push_back(tri);          // ajout du triangle dans la liste des triangles de l'quateur
			
	p2 = Point3D(x_centre+rayon_cercle_prec*cos(angle+ecart_angles),y_equateur,z_centre+rayon_cercle_prec*sin(angle+ecart_angles));
	tri = new Triangle(p1,p3,p2,couleur);
	res_int.push_back(tri);          // ajout du triangle dans la liste des triangles de l'quateur
				
	angle += ecart_angles;           // l'angle est incrment de ecart_angles
	j+=2;                            // j est incrment de 2 (nombre de triangles crs)
      }
    }
  }

  // projection sur la sphre des triangles obtenus
  // --> on calcul l'intersection entre la sphre et les rayons
  //     dont les origines sont les sommets des triangles et le vecteur directeur
  //     est le vecteur (0,1,0) (vecteur normal au plan de l'quateur)
  // On engendre donc les triangles d'un ple.
  // Les triangles de l'autre ple sont obtenus de la mme manire mais avec le vecteur normal (0,-1,0)
  
  Vecteur3D vect_normal_nord = Vecteur3D(0,1,0);                  // le vecteur normal "nord" au plan de l'quateur
  Vecteur3D vect_normal_sud = Vecteur3D(0,-1,0);                  // le vecteur normal "sud" au plan de l'quateur
  Rayon ray_p1_nord, ray_p2_nord, ray_p3_nord;                    // les rayons de l'intersection nord
  Rayon ray_p1_sud, ray_p2_sud, ray_p3_sud;                       // les rayons de l'intersection sud
  Triangle * tri_proj = NULL;                                	  // triangle projet correspondant
  Point3D * pA_nord = NULL, * pB_nord = NULL, * pC_nord = NULL;   // points intersections nord
  Point3D * pA_sud = NULL, * pB_sud = NULL, * pC_sud = NULL;      // points intersections sud
  
  // on projette chaque triangle de l'quateur
  for(int n=0;n<res_int.size();n++) {
    
    tri = res_int[n];                               // triangle de l'quateur en cours
    
    // gestion de l'hmisphre nord
	
    // cration des 3 rayons (1 pour chaque sommet du triangle)
    ray_p1_nord.setOrigine(tri->getPointA());
    ray_p1_nord.setVectDir(vect_normal_nord);
    ray_p2_nord.setOrigine(tri->getPointB());
    ray_p2_nord.setVectDir(vect_normal_nord);
    ray_p3_nord.setOrigine(tri->getPointC());
    ray_p3_nord.setVectDir(vect_normal_nord);
	
    // calcul des intersections
    pA_nord = ray_p1_nord.hitSphere(this);
    pB_nord = ray_p2_nord.hitSphere(this);
    pC_nord = ray_p3_nord.hitSphere(this);    
	
    // si une intersection est nulle, c'est que le point est situ sur le "bord" de l'quateur
    // --> on garde le point d'origine
    if(pA_nord==NULL) {
      pA_nord = new Point3D(tri->getPointA());
    }
    if(pB_nord==NULL) {
      pB_nord = new Point3D(tri->getPointB());
    }
    if(pC_nord==NULL) {
      pC_nord = new Point3D(tri->getPointC());
    }
    
    // on cre le triangle projet  partir des points intersections
    tri_proj = new Triangle(pA_nord,pC_nord,pB_nord,couleur);
    res.push_back(tri_proj);            // on ajoute ce triangle  la liste des triangles de la sphre
	
   
    // gestion de l'hmisphre sud
    
     // cration des 3 rayons (1 pour chaque sommet du triangle)
    ray_p1_sud.setOrigine(tri->getPointA());
    ray_p1_sud.setVectDir(vect_normal_sud);
    ray_p2_sud.setOrigine(tri->getPointB());
    ray_p2_sud.setVectDir(vect_normal_sud);
    ray_p3_sud.setOrigine(tri->getPointC());
    ray_p3_sud.setVectDir(vect_normal_sud);
	
    // calcul des intersections
    pA_sud = ray_p1_sud.hitSphere(this);
    pB_sud = ray_p2_sud.hitSphere(this);
    pC_sud = ray_p3_sud.hitSphere(this);
    
    // si une intersection est nulle, c'est que le point est situ sur le "bord" de l'quateur
    // --> on garde le point d'origine
    if(pA_sud==NULL) {
      pA_sud = new Point3D(tri->getPointA());
    }
    if(pB_sud==NULL) {
      pB_sud = new Point3D(tri->getPointB());
    }
    if(pC_sud==NULL) {
      pC_sud = new Point3D(tri->getPointC());
    }
    
    // on cre le triangle projet  partir des points intersections
    tri_proj = new Triangle(pA_sud,pB_sud,pC_sud,couleur);
    res.push_back(tri_proj);            // on ajoute ce triangle  la liste des triangles de la sphre
  }
    
  return res;             // on retourne la liste des triangles de la sphre
};
