/************************************************************************
*************************************************************************
****                                                                 ****
****                       Projet : LANCER DE RAYON                  ****
****                      fichier : sphere.h                         ****
****                                                                 ****
*************************************************************************
*************************************************************************
* Auteurs :                               date : 12/05/05               *
*          GUITTENY Fabrice                                             *
*          LE GOFF Erwan                                                *
*          LAHUEC Morgan                                                *
*************************************************************************
* Class: Sphere                                                         *
*                                                                       *
*************************************************************************
* Description :                                                         *
* Une sphere est representee par les 3 coordonnees de son centre, les 3 *
* composantes de sa couleur, son rayon, son indice miroir, son indice   *
* diffusion, son indice speculaire, son indice de rugosite, son type de *
* triangulation, son indice de triangulation.                           *
************************************************************************/

#ifndef _SPHERE_H
#define _SPHERE_H

#include"triangle.h"
#include"couleur.h"
#include"point.h"
#include<vector>
#include<iostream>
using namespace std;

class Sphere {		

 private :
	
  float rayon;					// rayon de la sphere
  Point3D centre;				// centre de la sphere
  float indMiroir;				// indice miroir compris entre 0 et 1
  float indDiffusion;				// indice de diffusion de la sphere compris entre 0 et 1
  float indSpeculaire;				// indice spculaire de la sphre compris entre 0 et 1
  int indRugosite;				// indice de rugosit de la sphre compris entre 0 et 100
  						// (plus c'est grand, plus la tache spculaire est petite)
  Couleur couleur;				// couleur de la sphere
	
  vector<Triangle*> listeTriangles;		// liste des triangles de la sphre (dans le cas d'une sphre triangule
  int indTriangulee;				// indice de triangulation de la sphre (entier pair)
                                                // -> correspond au nombre de mridiens de la sphre triangule
  int triangulee;                               // 0 : parfait, 1 : triangulation 1 (td), 2 : traingulation 2 (disque de l'equateur)

 public :

  // constructeurs
  Sphere() {};
  Sphere(Sphere * s) {
    centre=s->centre;
    rayon=s->rayon;
    indMiroir=s->indMiroir;
    indDiffusion=s->indDiffusion;
    indSpeculaire=s->indSpeculaire;
    indRugosite=s->indRugosite;
    couleur=s->couleur;
  };
  Sphere(const Point3D &c, const float &r) {
    centre=c;
    rayon=r;
    // indices par dfaut (arbitraires)
    indMiroir=0.0f;
    indDiffusion=0.3f;
    indSpeculaire=0.5f;
    indRugosite=5;
    couleur=Couleur();
  };
  Sphere(const Point3D &c, const float &r, const Couleur &coul) {
    centre=c;
    rayon=r;
    // indices par dfaut (arbitraires)
    indMiroir=0.0f;
    indDiffusion=0.3f;
    indSpeculaire=0.5f;
    indRugosite=5;
    couleur=coul;
  };		
  Sphere(const Point3D &c, const float &r, const float &im, const float &id, const float &is, const int &ir, const Couleur &coul) {
    centre=c;
    rayon=r;
    indMiroir=im;
    indDiffusion=id;
    indSpeculaire=is;
    indRugosite=ir;
    couleur=coul;
  };
  Sphere(const Point3D &c, const float &r, const float &im, const float &id, const float &is, const int &ir, const Couleur &coul, const int &typT,const int &it) {
    centre=c;
    rayon=r;
    indMiroir=im;
    indDiffusion=id;
    indSpeculaire=is;
    indRugosite=ir;
    couleur=coul;
    triangulee=typT;
    indTriangulee=it; 
    switch(typT){
    case 1 : listeTriangles=Triangule();break;
    case 2 : listeTriangles=TrianguleEquateur();break;
    default : break;
    }
  };

  // destructeur
  ~Sphere() {};

  // accesseurs
  Point3D getCentre() {return centre;};
  float getRayon() {return rayon;};
  float getIndMiroir() {return indMiroir;};
  float getIndDiffusion() {return indDiffusion;};
  float getIndSpeculaire() {return indSpeculaire;};
  int getIndRugosite() {return indRugosite;};
  Couleur getCouleur() {return couleur;};	
  vector<Triangle*> getListeTriangles() {return listeTriangles;};
  int getIndTriangulee() {return indTriangulee;};
  int getTriangulee() {return triangulee;};

  // modificateurs
  void setCentre(const Point3D &p) {centre=p;};
  void setRayon(const float &r) {rayon=r;};
  void setIndMiroir(const float &i) {indMiroir=i;};
  void setIndDiffusion(const float &i) {indDiffusion=i;};
  void setIndSpeculaire(const float &i) {indSpeculaire=i;};
  void setIndRugosite(const int &i) {indRugosite=i;};
  void setCouleur(const Couleur &coul) {couleur=coul;};
  void setListeTriangles(const vector<Triangle*> &v) {listeTriangles=v;};
  void setIndTriangulee(const int &i) {indTriangulee=i;};
  void setTriangulee(const int &typT){triangulee=typT;};

  // position d'un point par rapport  une sphre (fonction non utilise)
  int positionParRapportSphere(Point3D p);
  
  // affichage des informations de la sphre
  void afficherInfos();
  
  // la sphre est-elle triangule ?
  bool estTriangulee() {return (triangulee==1 || triangulee==2);};
  
  // fonction de triangulation de la sphre
  vector<Triangle*> Triangule();
  vector<Triangle*> TrianguleEquateur();
};

#endif
