/************************************************************************
*************************************************************************
****                                                                 ****
****                       Projet : LANCER DE RAYON                  ****
****                      fichier : srclumiere.h                     ****
****                                                                 ****
*************************************************************************
*************************************************************************
* Auteurs :                               date : 12/05/05               *
*          GUITTENY Fabrice                                             *
*          LE GOFF Erwan                                                *
*          LAHUEC Morgan                                                *
*************************************************************************
* Class: SrcLumiere                                                     *
*                                                                       *
*************************************************************************
* Description :                                                         *
* Une source de lumiere est represent par ses trois coordonnees, une    *   
* intensite et une couleur                                              *
************************************************************************/
#ifndef _SRCLUMIERE_H
#define _SRCLUMIERE_H

#include"point.h"
#include"couleur.h"
#include<iostream>
using namespace std;

class SrcLumiere {

private :

	Point3D pSource;			// point origine de la source lumineuse
	float intensite;			// indice d'intensit compris entre 0 (normal) et 1 (trs fort)
	Couleur couleur;			// couleur de la source

public :

	// constructeurs

	SrcLumiere() {};
	
	SrcLumiere(const SrcLumiere &sl) {
		pSource=sl.pSource;
		intensite=sl.intensite;
		couleur=sl.couleur;
	};
	
	SrcLumiere(const Point3D &pS) {
		pSource=pS;
		// source dfinie par un seul point -> source blanche d'intensit 1 par dfaut
		intensite=1.0f;
		couleur=Couleur(255,255,255);
	};
	
	SrcLumiere(const Point3D &pS, const float &i) {
		pSource=pS;
		// sans couleur en paramtre, la source est blanche par dfaut
		couleur=Couleur(255,255,255);
		intensite=i;
	};

	SrcLumiere(const Point3D &pS, const Couleur &coul) {
		pSource=pS;
		couleur=coul;
		// sans intensit en paramtre, la source est d'intensit 1 par dfaut
		intensite=1.0f;
	};

	SrcLumiere(const Point3D &pS, const float &i, const Couleur &coul) {
		pSource=pS;
		couleur=coul;
		intensite=i;
	};

	
	// destructeur
	~SrcLumiere() {};

	// accesseurs
	Point3D getPSource() {return pSource;};
	float getIntensite() {return intensite;};
	Couleur getCouleur() {return couleur;};

	// modificateurs
	void setPSource(const Point3D &pS) {pSource=pS;};
	void setIntensite(const float &i) {intensite=i;};
	void setCouleur(const Couleur &coul) {couleur=coul;};

	// affichage des infos
	void afficherInfos() {
		cout<<"- Point : ";
		pSource.afficherInfos();
		cout<<"- Intensite : "<<intensite<<endl;
		cout<<"- Couleur : ";
		couleur.afficherInfos();
	};
	
};

#endif
