/************************************************************************
*************************************************************************
****                                                                 ****
****                       Projet : LANCER DE RAYON                  ****
****                      fichier : vecteur.h                        ****
****                                                                 ****
*************************************************************************
*************************************************************************
* Auteurs :                               date : 12/05/05               *
*          GUITTENY Fabrice                                             *
*          LE GOFF Erwan                                                *
*          LAHUEC Morgan                                                *
*************************************************************************
* Class: Vecteur3D                                                      *
*                                                                       *
*************************************************************************
* Description :                                                         *
* Un vecteur3D possede trois coordonnees (x,y,z) .                      *
* Les operateur ont ete surdefini de maniere a pouvoir appliquer        *
* directement les operations tels que produits scalaire, vectoriel      *
************************************************************************/


#ifndef _VECTEUR3D_H
#define _VECTEUR3D_H

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include<iostream>
using namespace std;

class Vecteur3D {

private :
  
  float x;	// abscisse du vecteur
  float y;	// ordonne du vecteur
  float z;	// hauteur du vecteur
	
public :

  // constructeurs
  Vecteur3D() {x=0;y=0;z=0;};
  Vecteur3D(const float &cx, const float &cy, const float &cz) {x=cx;y=cy;z=cz;};
  Vecteur3D(Point3D p1, Point3D p2) {
    // p1 -> point de dpart	P2 -> point d'arrive
    x = p2.getX() - p1.getX();
    y = p2.getY() - p1.getY();
    z = p2.getZ() - p1.getZ();
  };
  
  // destructeur
  ~Vecteur3D() {};

  // accesseurs
  float getX() {return x;};
  float getY() {return y;};
  float getZ() {return z;};

  // modificateurs
  void setX(const float &cx) {x=cx;};
  void setY(const float &cy) {y=cy;};
  void setZ(const float &cz) {z=cz;};

  // afficher Infos
  void afficherInfos() {
    cout<<"- Vecteur ("<<x<<","<<y<<","<<z<<")"<<endl;
  };

  // addition de vecteurs
  Vecteur3D operator + (Vecteur3D vect) {
    return Vecteur3D(vect.getX()+x, vect.getY()+y, vect.getZ()+z);
  }

  // soustraction de vecteurs
  Vecteur3D operator - (Vecteur3D vect) {
    return Vecteur3D(x-vect.getX(), y-vect.getY(), z-vect.getZ());
  }

  // produit vectoriel
  Vecteur3D operator ^ (Vecteur3D vect) {
    Vecteur3D vNormal;
    vNormal.x = ((y * vect.getZ()) - (z * vect.getY()));
    vNormal.y = ((z * vect.getX()) - (x * vect.getZ()));
    vNormal.z = ((x * vect.getY()) - (y * vect.getX()));
    return vNormal;                                     
  }
    
  // multiplication par un scalaire
  Vecteur3D operator * (float num) {
    return Vecteur3D(x*num, y*num, z*num);
  }

  // produit scalaire
  float operator * (Vecteur3D vect) {
    return (x*vect.getX() + y*vect.getY() + z*vect.getZ());
  }

  // division par un scalaire
  Vecteur3D operator / (float num) {
    return Vecteur3D(x/num, y/num, z/num);
  }

  // affectation de vecteurs
  void operator = (Vecteur3D vect) {
    x = vect.getX();
    y = vect.getY();
    z = vect.getZ();
  }
  
  // galit de vecteurs
  bool operator == (Vecteur3D vect) {
    return (x==vect.getX() && y==vect.getY() && z==vect.getZ());
  }

  // diffrence de vecteurs
  bool operator != (Vecteur3D vect) {
    return (x!=vect.getX() || y!=vect.getY() || z!=vect.getZ());
  }
    
  // norme d'un vecteur
  float Norme() {
    // Norme = sqrt(x^2 + y^2 + z^2)
    return (float)sqrt( Vecteur3D(x,y,z) * Vecteur3D(x,y,z) );
  }
  
  // normalisation d'un vecteur
  Vecteur3D Normalise() {   
    return (Vecteur3D(x,y,z) / Norme());
  }
  
};

#endif
